@@ -118,7 +118,7 @@ context "Parser" do
     line = 'height=100,caption="",link="images/octocat.png"'
     expected = {'height' => '100', 'caption' => '', 'link' => 'images/octocat.png'}
     Asciidoctor::AttributeList.new(line).parse_into(attributes)
-    assert_equal expected, attributes 
+    assert_equal expected, attributes
   end
 
   test "collect named attribute single-quoted" do
@@ -134,7 +134,7 @@ context "Parser" do
     line = "height=100,caption='',link='images/octocat.png'"
     expected = {'height' => '100', 'caption' => '', 'link' => 'images/octocat.png'}
     Asciidoctor::AttributeList.new(line).parse_into(attributes)
-    assert_equal expected, attributes 
+    assert_equal expected, attributes
   end
 
   test "collect named attributes unquoted" do
@@ -477,6 +477,29 @@ Ryan Waldron
     assert_equal '2013-12-18', metadata['revdate']
   end
 
+  test 'parse rev number with trailing comma' do
+    input = <<-EOS
+Stuart Rackham
+v8.6.8,
+    EOS
+    metadata, _ = parse_header_metadata input
+    assert_equal 7, metadata.size
+    assert_equal '8.6.8', metadata['revnumber']
+    assert !metadata.has_key?('revdate')
+  end
+
+  # Asciidoctor recognizes a standalone revision without a trailing comma
+  test 'parse rev number' do
+    input = <<-EOS
+Stuart Rackham
+v8.6.8
+    EOS
+    metadata, _ = parse_header_metadata input
+    assert_equal 7, metadata.size
+    assert_equal '8.6.8', metadata['revnumber']
+    assert !metadata.has_key?('revdate')
+  end
+
   # while compliant w/ AsciiDoc, this is just sloppy parsing
   test "treats arbitrary text on rev line as revdate" do
     input = <<-EOS
@@ -516,7 +539,7 @@ Joe Cool
     EOS
     metadata, _ = parse_header_metadata input
     assert_equal 'Must start revremark-only line with space', metadata['revremark']
-    assert_equal '', metadata['revdate']
+    assert !metadata.has_key?('revdate')
   end
 
   test "skip line comments before author" do
@@ -576,7 +599,7 @@ v0.0.7, 2013-12-18
     assert_equal 'SJR', blankdoc.attributes['authorinitials']
   end
 
-  test 'reset block indent to 0' do
+  test 'adjust indentation to 0' do
     input = <<-EOS.chomp
     def names
 
@@ -594,11 +617,11 @@ end
     EOS
 
     lines = input.split("\n")
-    Asciidoctor::Parser.reset_block_indent! lines
+    Asciidoctor::Parser.adjust_indentation! lines
     assert_equal expected, (lines * "\n")
   end
 
-  test 'reset block indent mixed with tabs and spaces to 0' do
+  test 'adjust indentation mixed with tabs and spaces to 0' do
     input = <<-EOS.chomp
     def names
 
@@ -616,11 +639,31 @@ end
     EOS
 
     lines = input.split("\n")
-    Asciidoctor::Parser.reset_block_indent! lines
+    Asciidoctor::Parser.adjust_indentation! lines, 0, 4
     assert_equal expected, (lines * "\n")
   end
 
-  test 'reset block indent to non-zero' do
+  test 'expands tabs to spaces' do
+    input = <<-EOS.chomp
+Filesystem				Size	Used	Avail	Use%	Mounted on
+Filesystem              Size    Used    Avail   Use%    Mounted on
+devtmpfs				3.9G	   0	 3.9G	  0%	/dev
+/dev/mapper/fedora-root	 48G	 18G	  29G	 39%	/
+    EOS
+
+    expected = <<-EOS.chomp
+Filesystem              Size    Used    Avail   Use%    Mounted on
+Filesystem              Size    Used    Avail   Use%    Mounted on
+devtmpfs                3.9G       0     3.9G     0%    /dev
+/dev/mapper/fedora-root  48G     18G      29G    39%    /
+    EOS
+
+    lines = input.split("\n")
+    Asciidoctor::Parser.adjust_indentation! lines, 0, 4
+    assert_equal expected, (lines * "\n")
+  end
+
+  test 'adjust indentation to non-zero' do
     input = <<-EOS.chomp
     def names
 
@@ -631,39 +674,39 @@ end
 
     expected = <<-EOS.chomp
   def names
-  
+
     @name.split ' '
-  
+
   end
     EOS
 
     lines = input.split("\n")
-    Asciidoctor::Parser.reset_block_indent! lines, 2
+    Asciidoctor::Parser.adjust_indentation! lines, 2
     assert_equal expected, (lines * "\n")
   end
 
-  test 'preserve block indent' do
+  test 'preserve block indent if indent is -1' do
     input = <<-EOS
     def names
-    
+
       @name.split ' '
-    
+
     end
     EOS
 
     expected = input
 
     lines = input.lines.entries
-    Asciidoctor::Parser.reset_block_indent! lines, nil
+    Asciidoctor::Parser.adjust_indentation! lines, -1
     assert_equal expected, lines.join
   end
 
-  test 'reset block indent hands empty lines gracefully' do
+  test 'adjust indentation handles empty lines gracefully' do
     input = []
     expected = input
 
     lines = input.dup
-    Asciidoctor::Parser.reset_block_indent! lines
+    Asciidoctor::Parser.adjust_indentation! lines
     assert_equal expected, lines
   end
 
